Module dependencies

Module dependencies 

Expresssion of dependencies among Resources in a Comprehensive Assembly. The salient traits are:

Structs

MayFail
A wrapper for Vec<Arc<dyn Trait>> intended to be used as a ResourceDependency which says that some of the resource types implementing the trait are allowed to fail to initialise, and the rest should be retained.

Traits

ResourceDependencies
This trait expresses the collection of types of other resources that a Resource depends on. It is also used to list the top-level resource types at the roots of the Assembly graph. There are two main ways to use it:
ResourceDependency
Expresses a single type of dependency of one resource (or of the Assembly’s top level) on another.

Type Aliases

NoDependencies
Convenience type that can be used as the Dependencies associated type on any leaf Resource.

Derive Macros

ResourceDependencies
This macro should be used to derive the ResourceDependencies trait for expressing dependencies between resources.