Expresssion of dependencies among Resources in a Comprehensive
Assembly. The salient traits are:
ResourceDependency: Expresses a dependency of oneResourceon another one (or set of other ones) in anAssemblyResourceDependencies: A list ofResourceDependency.
Structs
- MayFail
- A wrapper for
Vec<Arc<dyn Trait>>intended to be used as aResourceDependencywhich says that some of the resource types implementing the trait are allowed to fail to initialise, and the rest should be retained.
Traits
- Resource
Dependencies - This trait expresses the collection of types of other resources that a
Resource depends on. It is also used to list the top-level resource
types at the roots of the
Assemblygraph. There are two main ways to use it: - Resource
Dependency - Expresses a single type of dependency of one resource (or of the
Assembly’s top level) on another.
Type Aliases
- NoDependencies
- Convenience type that can be used as the
Dependenciesassociated type on any leafResource.
Derive Macros
- Resource
Dependencies - This macro should be used to derive the
ResourceDependenciestrait for expressing dependencies between resources.